package pep;

import java.io.*; import java.util.*;
import static pep.Main.nSubjects; import static pep.Main.numPerVar;
import static pep.Main.multTarg;  import static pep.Main.multDist;
import static pep.Main.multCue;

/**This class computes participant mean RT and percentage error, then prints
 * results to Means.txt. It can be run independently of the full model.*/
public class DataAnalysis{
 //----------VARIABLES--------------------------------------------------
 /**file name*/              static String fileName;
 /**number of variables*/    static int nVariables = numPerVar.length;
 //---------------------------------------------------------------------
 /**number of conditions*/   static int nConditions;
 /**condition number*/       static int condition;
 /**response time*/          static int rt;
 /**sum of response times*/  static int[] sumRT;
 /**number correct*/         static int[] nRT;
 /**mean response time*/     static int[] xRT;
 /**accuracy*/               static int accuracy;
 /**sum of accuracy scores*/ static int[] sumACC;
 /**number of responses*/    static int nACC[];
 /**percentage error*/       static double[] xERR;
 //-------------------------------------------------------------------
 /**check sequence?*/        static boolean isCheckSeq = Main.isCheckSeq;
 /**filter repetitions?*/    static boolean isTrimmed = Main.isTrimmed;
 /**check all rep types*/    static boolean isAllSeq = Main.isAllSeq;
 /**cued task switching?*/   static boolean isCueSwitch = Main.isCueSwitch;
 /**only switch cost?*/      static boolean onlySwitch = Main.onlySwitch;
 /**only cue factor?*/       static boolean onlyCue = Main.onlyCue;
 /**response repetition?*/   static boolean onlyResponse = Main.onlyResponse;
 /**switch plus congruency*/ static boolean andCong = Main.andCong;
 /**switch only congruency*/ static boolean onlyCong = Main.onlyCong;
 /**trim previous errors*/   static boolean prevError = Main.prevError;
 //--------------------------------------------------------------------
 /**distracter number*/      static int[] dist;
 /**target number*/          static int[] targ;
 /**cue number*/             static int[] cue;
 /**previous distracter*/    static int n1dist;
 /**previous target*/        static int n1targ;
 /**response number*/        static int resp;
 /**previous response*/      static int n1resp;
 /**previous cue*/           static int n1cue;
 /**task number*/            static int task;
 /**previous task*/          static int n1task;
 /**congruency*/             static int cong;
 /**previous congruency*/    static int n1cong;
 /**previous accuracy*/      static int n1acc;
 /**count null responses*/   static int missed;
 /**count all response*/     static int count;

 /**Clears arrays and calls getMeans() for each participant.
  * @param args Not used.*/
 public static void main(String[] args){
  checkValidSelections();
  clear();
  nConditions = 1;
  for(int k=0; k<nVariables; k++){
   nConditions *= (numPerVar[k]);
  }
  if(isAllSeq) nConditions = 15;
  else if(isCheckSeq) nConditions*=2;
  else if(isCueSwitch){
   if(onlySwitch) nConditions = 2;
   else if(onlyCue) nConditions = 3;
   else if(onlyResponse) nConditions = 4;
   else if(andCong) nConditions = 18;
   else nConditions = (onlyCong) ? 4 : 10;
  }
  getMeans();
 }

 /**Opens a participant datafile and calculates their condition means.*/
 public static void getMeans(){
  BufferedReader in = null;
  String[] trialString = null;
  int currSubj;
  try{
   fileName = "Dataset.txt";
   in = new BufferedReader(new FileReader(fileName));
  }
  catch(FileNotFoundException e){
   System.out.println("Error setting input stream.");
  }
  try{
   trialString = in.readLine().split("\\t");
  }
  catch(IOException e){
   System.out.println("No data in file.");
  }
  for(int i=1; i<=nSubjects; i++){
   currSubj = Integer.parseInt(trialString[0]);
   for(int v=0; v<nVariables; v++){
    sumRT = new int[nConditions];  sumACC = new int[nConditions];
    nRT = new int[nConditions];    nACC = new int[nConditions];
    xRT = new int[nConditions];    xERR = new double[nConditions];
    missed = 0; count = 0;
   }
   boolean doneSubj=false;
   while(!doneSubj&&in!=null){
    try{
     condition=0;
     int offset = 2; //subject and actual response not relevant    
     for(int c=0; c<multCue; c++,offset++)
      cue[c] = Integer.parseInt(trialString[offset]);
     for(int t=0; t<multTarg; t++,offset++)
      targ[t] = Integer.parseInt(trialString[offset]);
     for(int d=0; d<multDist; d++,offset++)
      dist[d] = Integer.parseInt(trialString[offset]);
     int orthogCount = 1;
     for(int n=0; n<nVariables; n++,offset++){
      condition += (Integer.parseInt(trialString[offset])*orthogCount);
      orthogCount *= (numPerVar[n]);
     }
     task = (int)((condition) / 2);
     cong = (Integer.parseInt(trialString[offset-nVariables]));
     if(n1cong<999 & isCheckSeq) condition += n1cong*orthogCount;
     resp = Integer.parseInt(trialString[offset++]);
     rt = Integer.parseInt(trialString[offset++]);
     accuracy = Integer.parseInt(trialString[offset]);
     if(isAllSeq) fullSequence();
     else if(isCueSwitch) cueSequence();
     boolean isValid = (!isTrimmed) ? true : trim();
     int actualResp = Integer.parseInt(trialString[1]);
     if(actualResp>700){
      isValid = false;
      missed++;
     }
     count++;
     if(prevError && n1acc!=1) isValid = false;
     if(isValid){
      sumACC[condition] += accuracy;
      nACC[condition]++;
      if(accuracy==1){
       sumRT[condition] += rt;
       nRT[condition]++;
      }
     }
     if(multDist>0) n1dist = dist[0];
     n1targ = targ[0]; n1cong = cong;
     if(multCue>0) n1cue = cue[0];
     n1resp = resp;
     n1acc = accuracy;
     if(Main.nGoals>0) n1task = task;
     String nextTrial = in.readLine();
     if(nextTrial==null ||nextTrial.isEmpty()) in=null;
     else{
      trialString = nextTrial.split("\\t");
      if(currSubj != Integer.parseInt(trialString[0])) doneSubj=true;
     }
    }
    catch(NumberFormatException | IOException e3){
     in = null;
     System.out.println("Problem reading data file.");
    }
   }
   printToFile(i);
  }
 }

 /**Runs a full CSE sequence analysis (15 conditions).*/
 private static void fullSequence(){
  if(n1cong==1){
   if(cong==1){
    condition = (n1targ==dist[0]) ? (n1dist==targ[0] ? 14 : 13) :
    (n1dist==targ[0] ? 12 : (n1targ==targ[0] ?
    (n1dist==dist[0] ? 11 : 10) : (n1dist==dist[0] ? 9 : 8)));
   }
   else condition = (n1targ==targ[0]) ? 7 : (n1dist==dist[0] ? 6 : 5);
  }
  else{
   if(cong==1)
    condition = (n1targ==targ[0]) ? 4 : (n1dist==dist[0] ? 3 : 2);
   else condition = (n1targ==targ[0]) ? 1 : 0;
  }
 }

 /**Runs a full task switch sequence analysis (10 conditions).*/
 private static void cueSequence(){
  if(onlySwitch){
   condition = (n1task==task) ? 0 : 1;
  }
  else if(onlyCue){
   if(n1cue==cue[0]) condition = 0;
   else condition = (n1task==task) ? 1 : 2;
  }
  else if(onlyResponse){
   if(n1task==task) condition = (n1resp==resp) ? 0 : 1;
   else condition = (n1resp==resp) ? 2 : 3;
  }
  else if(andCong){
   if(cong==0){
    if(n1cue==cue[0]){
     if(n1targ==targ[0]) condition = 0;
     else condition = (n1resp==resp) ? 2 : 4;
    }
    else if(n1task==task){
     if(n1targ==targ[0]) condition = 6;
     else condition = (n1resp==resp) ? 8 : 10;
    }
    else if(n1targ==targ[0]) condition = 12;
    else condition = (n1resp==resp) ? 14 : 16;
   }
   else{
    if(n1cue==cue[0]){
     if(n1targ==targ[0]) condition = 1;
     else condition = (n1resp==resp) ? 3 : 5;
    }
    else if(n1task==task){
     if(n1targ==targ[0]) condition = 7;
     else condition = (n1resp==resp) ? 9 : 11;
    }
    else if(n1targ==targ[0]) condition = 13;
    else condition = (n1resp==resp) ? 15 : 17;
   }
  }
  else if(onlyCong){
   if(cong==0) condition = (n1cue==cue[0]) ? 0 : 2;
   else condition = (n1cue==cue[0] ? 1 : 3);
  }
  else{
   if(n1cue==cue[0]){
    if(n1targ==targ[0]) condition = 0;
    else condition = (n1resp==resp) ? 1 : 2;
   }
   else if(n1task==task){
    if(n1targ==targ[0]) condition = 3;
    else condition = (n1resp==resp) ? 4 : 5;
   }
   else if(n1targ==targ[0]) condition = (n1resp==resp) ? 6 : 7;
   else condition = (n1resp==resp) ? 8 : 9;
  }
 }

 /**Trims stimulus repetitions.
  * @return Kept or not?*/
 private static boolean trim(){
  if(n1targ!=targ[0] & n1dist!=dist[0]){
   if(Main.nConflicting>0)
    return (n1targ!=dist[0] & n1dist!=targ[0]);
   else return true;
  }
  else return false;
 }

 /**Sets starting values for all variables.*/
 private static void clear(){
  dist = new int[multDist];
  targ = new int[multTarg];
  cue = new int[multCue];
  Arrays.fill(dist,999);
  Arrays.fill(targ,999);
  Arrays.fill(cue,999);
  n1dist = 999;
  n1targ = 999;
  resp = 999;
  n1resp = 999;
  n1cue = 999;
  task = 999;
  n1task = 999;
  cong = 999;
  n1cong = 999;
  n1acc = 999;
 }

 /**Prints means to file.
  * @param i Participant number*/
 private static void printToFile(int i){
  FileOutputStream out = null;
  PrintStream p = null;
  try{
   if(i>1) out = new FileOutputStream("Means.txt", true);
   else out = new FileOutputStream("Means.txt");
   p = new PrintStream(out);
  }
  catch(FileNotFoundException e){
   System.out.println("Error setting output stream.");
  }
  for(int k=0; k<nConditions; k++){
   if(nRT[k]!=0){
    xRT[k] = sumRT[k]/nRT[k];
    p.print(xRT[k]+"\t");
   }
   else p.print("\t");
  }
  for(int k=0; k<nConditions; k++){
   if(nACC[k]!=0){
    xERR[k] = (1-((double)sumACC[k]/nACC[k]))*100;
    p.print(xERR[k]+"\t");
   }
   else p.print("\t");
  }
  p.println((missed*100)/count);
  try{
   p.close(); out.close();
  }
  catch(IOException e2){
   System.out.println("Error closing output stream");
  }
 }

 /**Checks to make sure data analysis selections are valid.*/
 private static void checkValidSelections(){
  if((andCong||onlyCong)&&!isCueSwitch){
   System.out.print("Error: You cannot check task-rule congruency");
   System.out.println(" without selecting isCueSwitch.");
   System.out.println("isCueSwitch set to true.");
   isCueSwitch = true;
  }
  if(andCong&&onlyCong){
   System.out.print("Error: You cannot set both andCong and onlyCong");
   System.out.println(" to true.");
   System.out.println("onlyCong set to false.");
   onlyCong = false;
  }
  if(isTrimmed&&isAllSeq){
   System.out.print("Error: You cannot set both isAllSeq and");
   System.out.println(" isTrimmed to true.");
   System.out.println("isTrimmed set to false.");
   isTrimmed = false;
  }
  if(isTrimmed&&isCueSwitch){
   System.out.print("Error: You cannot set both isCueSwitch and");
   System.out.println(" isTrimmed to true.");
   System.out.println("isTrimmed set to false.");
   isTrimmed = false;
  }
  if(isAllSeq&&isCueSwitch){
   System.out.print("Error: You cannot set both isAllSeq and isCueSwitch");
   System.out.println(" to true.");
   System.out.println("isAllSeq set to false.");
   isAllSeq = false;
  }
 }
}