package pep;

import java.io.*; import java.util.*;
import static pep.Main.nBlocks;  import static pep.Main.isReplace;
import static pep.Main.nSamples; import static pep.Main.isPaired;
import static pep.Main.twoSets;

/**This class creates the randomized trial order for each participant.*/
public class DataSet{
 /**number of task phases*/   private int nPhases;
 /**file reader*/             private BufferedReader br;
 /**trial counter*/           private int nTrials = 0;
 /**random number generator*/ private Random r = new Random();
 /**final array of trials*/   private String set[];
 /**loop counter*/            private int counter;
 /**task phase counter*/      private int phase = 0;
 /**array of trials*/         private String trials[][];
 /**second trial array*/      private String trialsB[][];

 /**This method uses a source file to read what trials occur in a block,
  * randomizes them, and saves the complete sequence of trials to an array.
  * @param source The name of the text file containing the task stimuli.
  * @param subj The participant number.*/
 DataSet(String source,int subj){
  checkValidSelections();
  nPhases = (isReplace) ? nSamples.length : nBlocks.length;
  set = new String[1002];
  trials = new String[nPhases][1002];
  trialsB = new String[nPhases][1002];
  try{
   br = new BufferedReader(new FileReader(source));
   counter = 0;
   for(boolean isFileEnd=false; !isFileEnd; nTrials++){
    trials[phase][nTrials] = br.readLine();
    switch(trials[phase][nTrials]){
     case "END_OF_FILE":
      isFileEnd=true;
      if(twoSets || isPaired) splitTrials();
      mixTrials();
      break;
     case "END_OF_BLOCK":
      if(twoSets || isPaired) splitTrials();
      mixTrials();
      nTrials=-1;
      phase++;
      break;
    }
   }
   set[counter] = "END_OF_FILE";
   br.close();
  }
  catch(IOException e){
   System.out.println("Error reading source");
  }
 }

 /**Randomizes the trial set with one of several randomization routines.*/
 private void mixTrials(){
  if(isReplace){
   for(int sample=0; sample<nSamples[phase]; sample++){
    int randNum = r.nextInt(nTrials);
    set[counter] = trials[phase][randNum];
    counter++;
    if(isPaired){
     set[counter] = trialsB[phase][randNum];
     counter++;
    }
    else if(twoSets){
     randNum = r.nextInt(nTrials);
     set[counter] = trialsB[phase][randNum];
     counter++;
    }
   }
  }
  else{ //random without replacement
   for(int block=0; block<nBlocks[phase]; block++){
    for(int trial=0; trial<nTrials; trial++){
     int randNum = r.nextInt(nTrials-trial);
     set[counter] = trials[phase][randNum];
     counter++;
     String temp = trials[phase][randNum];
     trials[phase][randNum] = trials[phase][nTrials-trial-1];
     trials[phase][nTrials-trial-1] = temp;
     if(isPaired){
      set[counter] = trialsB[phase][randNum];
      counter++;
      temp = trialsB[phase][randNum];
      trialsB[phase][randNum] = trialsB[phase][nTrials-trial-1];
      trialsB[phase][nTrials-trial-1] = temp;
     }
     else if(twoSets){
      randNum = r.nextInt(nTrials-trial);
      set[counter] = trialsB[phase][randNum];
      counter++;
      temp = trialsB[phase][randNum];
      trialsB[phase][randNum] = trialsB[phase][nTrials-trial-1];
      trialsB[phase][nTrials-trial-1] = temp;
     }
    }
   }
  }
 }

 /**This method retrieves the indexed trial.
  * @param index The trial number.
  * @return The trail string.*/
 public String get(int index){
  return set[index];
 }

 /**Splits trials into two alternating stimulus sets.*/
 private void splitTrials(){
  nTrials = ((nTrials+1) / 2); //half
  for(int i=0; i<nTrials; i++){
   trialsB[phase][i] = trials[phase][nTrials+i];
  }
 }

 /**Checks to make sure dataset selections are valid.*/
 private static void checkValidSelections(){
  if(twoSets&&isPaired){
   System.out.print("Error: You cannot set both twoSets and isPaired");
   System.out.println(" to true.");
   System.out.println("twoSets set to false.");
   twoSets = false;
  }
 }
}