package pep;

import java.io.*; import java.util.*;

public class Evolve{
 /**use saved sample?*/      private static final boolean savedSample = true;
 /**retest saved sample?*/   private static final boolean retestSaved = false;
 /**number of generations*/  private static final int maxGenerations = 1000;
 /**population size*/        private static final int population = 40;
 /**breeding width*/         private static final int neighbourhood = 3;
 /**criterion for ending*/   private static final double criterion = 100;
 /**mutation rate*/          private static final double rate = .1;
 /**mutation extremity*/     private static final double extremity = .025;
 /**weight importance*/      private static final double wImportance = 0.0;
 /**error data weight*/      private static final double errorRateWeight = 20;
 /**break every x sims*/     private static final int breakInterval = 10;
 /**break duration (sec)*/   private static final int breakTime = 5;
 /**initial parameter seed*/
 private static final double[] evoSeed = {};
 /**parameter maximum limits*/
 private static final double[] maxParam = {};
 /**response time tests, see notes*/
 private static final int[] testsRT = {458,470,588,631,673,
                                       724,660,693,780,791,
                                       781,859,858,935,905,
                                       964,852,928};
 /**error rate tests, see notes*/
 private static final double[] testsErr = {0,0.4,1.6,7.4,6.1,
                                           10.7,4.2,5.2,4.2,11.2,
                                           5,10.3,7.2,19.4,6.6,
                                           20,3.9,12.2};
 /**number of parameters tested*/
 private static final int nParam = evoSeed.length;
 /**parent population*/
 private static final double[][] parents = new double[population][nParam];
 /**new child*/
 private static final double[] child = new double[nParam];
 /**current sample to test*/
 public static double[] sample = evoSeed.clone();
 /**parent error*/
 private static final double[] parentError = new double[population];
 /**child error*/             private static double childError;
 /**maximum error*/           private static double maxError;
 /**worst parent*/            private static int targetParent;
 /**stopping control*/        private static boolean finished = false;
 /**random number generator*/ private static final Random r = new Random();
 /**nondecision time*/        private static int nondecision;
 /**counter (for breaks)*/    private static int breakCount = 0;
 /**parent nondecision times*/
 private static final int[] parentNDT = new int[population];
 /**number of columns in file*/
 private static final int nColumns = testsRT.length+testsErr.length;
 /**weight for each cell in error calculation*/
 private static final double[] importance = new double[nColumns];
 /**squared error for each cell*/
 private static final double[] errors = new double[nColumns];
  /**number of RT columns in file*/
 private static final int rtColumns = testsRT.length;
 /**column sums for computing means*/
 private static final double[] sums = new double[nColumns];
 /**column counts for computing means*/
 private static final int[] counts = new int[nColumns];
 /**column means*/
 private static double[] means = new double[nColumns];
 /**generation counter*/ private static int generation = 0;
 /**test mean*/          private static int xTest = 0;
 /**sample mean*/        private static int xSample = 0;
 /**rate parent murder*/ private static double murderRate = 1;
 /**means computed?*/    private static boolean filled = false;
 /**parent means*/
 private static final double[][] parentMeans=new double[population][nColumns];
 /**null response count*/   private static int missed;
 /**parent miss rates*/
  private static final int[] missRates = new int[population];
 /**punish null responses*/
 private static final int punishNull = testsErr.length*5;

 /**Main method for creating populations and running Main.java.
  * @param args Not used.
  * @throws java.lang.InterruptedException*/
 public static void main(String[] args) throws InterruptedException{
  testStats();
  Arrays.fill(importance,1.0);
  if(savedSample){
   readParents();
   if(retestSaved) runParents();
  }
  else{
   seedParents();
   runParents();
  }
  while(!finished){
   breed();
   runParents();
   if(maxError<criterion) finished=true;
  }
  System.out.println();
  System.out.println("End of simulation");
  System.out.println("Maximum error: "+(int)maxError);
 }

 /**Reads a saved sample of parents and saves to array.*/
 private static void readParents(){
  BufferedReader in;
  try{
   in = new BufferedReader(new FileReader("Sample.txt"));
   for(int pop=0; pop<population; pop++){
    String[] tempArray = in.readLine().split("\\t");
    for(int param=0; param<evoSeed.length; param++){
     parents[pop][param] = Double.parseDouble(tempArray[param]);
    }
    int offset = evoSeed.length;
    parentNDT[pop] = Integer.parseInt(tempArray[offset]);
    offset++;
    parentError[pop] = Double.parseDouble(tempArray[offset]);
    offset++;
    for(int col=0; col<nColumns; col++)
     parentMeans[pop][col] = Double.parseDouble(tempArray[offset+col]);
    missRates[pop] = Integer.parseInt(tempArray[tempArray.length-1]);
   }
   setImportance();
   System.out.println("Saved sample maximum error: "+(int)maxError);
  }
  catch(IOException | NumberFormatException e){
   System.out.println("Error reading Sample.txt.");
  }
 }

 /**Creates parents using mutations of initial seed parameters.*/
 private static void seedParents(){
  for(int pop=0; pop<population; pop++){
   for(int param=0; param<nParam; param++){
    parents[pop][param] = mutate(evoSeed[param],param);
   }
  }
 }

 /**Create child from random and mutated parent traits.*/
 private static void seedChild(){
  int breedLocation = r.nextInt(population); //reference parent
  findMaxError(breedLocation);
  for(int param=0; param<nParam; param++){
   int selected = breedLocation + r.nextInt(neighbourhood); //select parent
   if(selected>=population) selected -= population; //range to population
   double trait = parents[selected][param];
   child[param] = mutate(trait,param);
  }
 }

 /**Run parents.*/
 private static void runParents() throws InterruptedException{
  for(int pop=0; pop<population; pop++){
   sample = parents[pop].clone();
   Main.main(new String[0]);
   missed = 0;
   parentError[pop] = findError()+(missed*punishNull);
   saveMeans(pop);
   System.out.print("Parent "+(pop+1)+" complete!");
   System.out.println(" Error: "+(int)parentError[pop]);
   breakCheck();
  }
  setImportance();
  printToFile();
  murderRate = 1;
 }

 /**Runs child samples until criterion or max run length met.*/
 private static void breed() throws InterruptedException{
  while(!finished && maxError>criterion){
   seedChild();
   sample = child.clone();
   Main.main(new String[0]);
   missed = 0;
   childError = findError()+(missed*punishNull);
   printProgress();
   generation++;
   if(generation>=maxGenerations) finished=true;
   System.out.print("Generation "+generation+" complete!");
   System.out.println(" Error: "+(int)childError);
   if(childError<maxError){
    System.out.print("Assassination attempt... ");
    Main.main(new String[0]); //retest
    missed = 0;
    childError = findError()+(missed*punishNull);
    System.out.println("New error: "+(int)childError);
    if(childError<maxError) killParent();
    else killChild();
   }
   else killChild();
   if(lowMurderRate()) runParents();
   breakCheck();
   findMaxError();
  }
 }

 /**Mutate selected trait relative to original parameter.
  * @param trait Trait parameter.
  * @param param Parameter number.
  * @return Mutated parameter.*/
 private static double mutate(double trait,int param){
  double newTrait = trait;
  if(r.nextDouble()<rate)
   newTrait += r.nextGaussian()*evoSeed[param]*extremity;
  if(newTrait>maxParam[param]) newTrait = maxParam[param];
  return (newTrait<0) ? 0 : newTrait;
 }

 /**Compute the error between observed results and criteria.
  * @return Error.*/
 private static double findError(){
  boolean murder = false;
  if(!filled) murder = fillColumns();
  double totalError = (murder) ? 10000000 : 0;
  if(!murder){
   sampleStats();
   for(int col=0; col<rtColumns; col++){
    double test = testsRT[col]-means[col]; //expected-observed
    test -= nondecision; //nondecision time
    errors[col] = (test*test);
    totalError += errors[col]*importance[col];
   }
   for(int col=rtColumns; col<nColumns; col++){
    double test = testsErr[col-rtColumns]-means[col];
    errors[col] = (test*test*errorRateWeight); //scale adjustment
    totalError += errors[col]*importance[col];
   }
  }
  return (totalError/nColumns);
 }

 /**Fills columns with data for error calculation.
  * @return Whether or not empty cells exist.*/
 private static boolean fillColumns(){
  Arrays.fill(sums,0.0);
  Arrays.fill(counts,0);
  Arrays.fill(means,0.0);
  BufferedReader in = null;
  String trialString[] = null;
  String fileName;
  try{
   fileName = "Means.txt";
   in = new BufferedReader(new FileReader(fileName));
  }
  catch(FileNotFoundException e){
   System.out.println("Error reading Means.txt.");
  }
  for(int x=0; x<Main.nSubjects; x++){
   try{
    trialString = in.readLine().split("\\t");
   }
   catch(IOException e){
    System.out.println("No data or not enough data in file.");
   }
   double[] tempArray = new double[nColumns];
   for(int col=0; col<nColumns; col++){
    try{
     tempArray[col] = Double.parseDouble(trialString[col]);
    }
    catch(NumberFormatException e){
     tempArray[col] = -1;
    }
   }
   missed += Integer.parseInt(trialString[nColumns]);
   for(int col=0; col<nColumns; col++){
    if(tempArray[col]>-1){ //non blank cell
     sums[col] += tempArray[col];
     counts[col]++;
    }
   }
  }
  boolean murder=false;
  for(int col=0; col<nColumns; col++){
   if(counts[col]>0) means[col] = ((double)sums[col]/counts[col]);
   else murder=true;
  }
  missed/=Main.nSubjects;
  return murder;
 }

 /**Computes the mean of the participant response times.*/
 private static void testStats(){
  xTest=0;
  for(int col=0; col<rtColumns; col++) xTest+=testsRT[col];
  xTest/=rtColumns;
 }

 /**Computes the mean and nondecision time of the current sample.*/
 private static void sampleStats(){
  xSample=0;
  for(int col=0; col<rtColumns; col++) xSample+=means[col];
  xSample/=rtColumns;
  nondecision = (xTest-xSample);
  if(nondecision < 0) nondecision = 0;
 }

 /**Compute which parent has worst error.*/
 private static void findMaxError(){
  maxError = parentError[0];
  targetParent = 0;
  for(int pop=1; pop<population; pop++){
   if(parentError[pop]>maxError){
    maxError = parentError[pop];
    targetParent = pop;
   }
  }
 }

 /**Compute which parent has worst error.
  * @param start First parent in subpopulation.*/
 private static void findMaxError(int start){
  maxError = parentError[start];
  targetParent = start;
  for(int pop=0,next=start; pop<neighbourhood; pop++,next++){
   if(next>=population) next -= population;
   if(parentError[next]>maxError){
    maxError = parentError[next];
    targetParent = next;
   }
  }
 }

 /**Kill worst parent, replace with child, and compute new worst parent.*/
 private static void killParent(){
  parents[targetParent] = child.clone();
  parentError[targetParent] = childError;
  saveMeans(targetParent);
  setImportance();
  printToFile();
  System.out.print(" \u2620 Parricide! ");
  System.out.println("Max error: "+(int)maxError+" \u2620");
 }

 /**Kill the child.*/
 private static void killChild(){
  //because reasons
 }

 /**Computes whether to retest parents after slow growth.*/
 private static boolean lowMurderRate(){
  murderRate*=.95;
  if(childError<=maxError) murderRate+=.05;
  return (murderRate<.01);
 }

 /**Checks whether a CPU break is needed.*/
 private static void breakCheck() throws InterruptedException {
  breakCount++;
  if(breakCount>=breakInterval){
   System.out.print("So sleepy. "+breakTime+" second nap...");
   Thread.sleep(breakTime*1000); //convert to milliseconds
   System.out.println(" Awake.");
   breakCount=0;
  }
 }

 /**Saves means of population member for error target computation.
  * @param target Targeted parent to save over.*/
 private static void saveMeans(int target){
  parentMeans[target] = means.clone();
  parentNDT[target] = nondecision;
  missRates[target] = missed;
 }

 /**Computes the adjusted importance of each cell.*/
 private static void setImportance(){
  Arrays.fill(means,0.0);
  Arrays.fill(importance,1.0);
  for(int col=0; col<nColumns; col++){
   for(int pop=0; pop<population; pop++) means[col] += parentMeans[pop][col];
   means[col] /= population;
  }
  filled = true;
  missed = 0;
  double totError = findError()*nColumns;
  for(int col=0; col<nColumns; col++){
   importance[col] *= (1-wImportance);
   importance[col] += (errors[col]*nColumns/totError)*wImportance;
  }
  for(int pop=0; pop<population; pop++){
   means = parentMeans[pop].clone();
   missed = missRates[pop];
   parentError[pop] = findError()+(missed*punishNull);
  }
  filled = false;
  findMaxError();
 }

 /**Print the weights, nondecision times, and errors to Sample.txt.*/
 private static void printToFile(){
  FileOutputStream out;
  PrintStream p = null;
  try{
   out = new FileOutputStream("Sample.txt");
   p = new PrintStream(out);
  }
  catch(FileNotFoundException e){
   System.out.println("Error setting output stream.");
  }
  for(int pop=0; pop<population; pop++){
   for(int param=0; param<nParam; param++) p.print(parents[pop][param]+"\t");
   p.print(parentNDT[pop]+"\t"+parentError[pop]);
   for(int col=0; col<nColumns; col++) p.print("\t"+parentMeans[pop][col]);
   p.println("\t"+missRates[pop]);
  }
 }

 /**Print the error of each child to Progress.txt.*/
 private static void printProgress(){
  FileOutputStream out;
  PrintStream p;
  try{
   if(generation>0) out = new FileOutputStream("Progress.txt", true);
   else out = new FileOutputStream("Progress.txt");
   p = new PrintStream(out);
   p.println(childError);
  }
  catch(FileNotFoundException e){
   System.out.println("Error writing progress to file.");
  }
 }
}