package pep;

/**This class defines the operations of Input nodes.*/
public class Input extends SubNode{
 /**stimulus bias*/    private double bias;
 /**stimulus signal*/  private double signal;
 /**bias change rate*/ private final double change;
 /**bias change rate*/ private final double mask = resolutionAdjust(.995);
 /**total amount memory retrieved*/
 public static ThreadLocal<Double> totRetrCue = new ThreadLocal<>();
 /**total amount memory retrieved*/
 public static ThreadLocal<Double> totRetrTarg = new ThreadLocal<>();
 /**total amount memory retrieved*/
 public static ThreadLocal<Double> totRetrDist = new ThreadLocal<>();
 
 /**Creates a new Input node.
  * @param subject Subject number used for static variable array.
  * @param isTarget Is it a target node?
  * @param isCue Is it a cue node?*/
 public Input(int subject, boolean isTarget, boolean isCue){
  super(subject);
  threshold = .5;
  bias = 0;
  signal = 0;
  change = resolutionAdjust(.034);
  if(isTarget&&isCue)
   System.out.println("Error: Input nodes cannot be targets and cues.");
  if(isTarget) strength = 3.4;
  else if(isCue) strength = 1.14;
  else strength = 1;
  nConnections = Main.nDecisions;
  weight = new double[nConnections];
 }

 /**Defines the input values for the node.
  * @param rate Amount of bias for the stimulus.
  * @param sig Amount of signal for the stimulus.*/
 public void turnOn(double rate,double sig){
  setBias(rate);
  signal = sig;
 }

 /**Computes a normally distributed starting bias value.
  * @param rate Bias multiplier.*/
 private void setBias(double rate){
  bias = addNoise(.8)*rate;
 }

 /**Activate node and send activation to Decision nodes.*/
 public void activate(){
  setInput();
  update();
  actOut();
 }

 /**Computes memory retrieval for cues with SubNode method.*/
 public void retrieveCue(){
  double total = totRetrCue.get();
  double multiplier = Executive.goalSearch.get();
  retrieve(total,multiplier);
 }

 /**Computes memory retrieval for targets.*/
 public void retrieveTarg(){
  double tot = totRetrTarg.get();
  double multiplier = Executive.stimSearch.get();
  retrieve(tot,multiplier);
 }

 /**Computes memory retrieval for distracters.*/
 public void retrieveDist(){
  double total = totRetrDist.get();
  double multiplier = Executive.nonSearch.get();
  retrieve(total,multiplier);
 }

 /**Updates bias and sets the input for cycle.*/
 private void setInput(){
  bias = (bias*(1-change)) + (signal*change);
  input += bias;
  input += addNoise(.012);
 }

 /**Reduces signal when stimulus removed from display.*/
 public void remove(){
  signal *= mask;
 }

 /**Reduces signal when stimulus is masked.*/
 public void mask(){
  signal = 0.0;
 }

 /**Shuts off signal and bias.*/
 public void shutOff(){
  signal = 0.0;
  bias = 0.0;
 }

 /**Sends activation to Decision node.*/
 private void actOut(){
  double output = (activation-threshold);
  if(output>0){
   for(int d= 0; d<nConnections; d++)
    Main.runs[subj].decisions[d].send(output*weight[d]);
  }
 }

 /**Clears totRetrieved values for the next cycle.*/
 public static void clear(){
  totRetrCue.set(0.0);
  totRetrTarg.set(0.0);
  totRetrDist.set(0.0);
 }
}