package pep;

import java.io.*;                  import java.util.*;
import static pep.Main.nTargets;   import static pep.Main.nDistracters;
import static pep.Main.multTarg;   import static pep.Main.multDist;
import static pep.Main.multCue;    import static pep.Main.nConflicting;
import static pep.Main.nCues;      import static pep.Main.nDecisions;
import static pep.Main.nGoals;     import static pep.Main.nResponses;
import static pep.Main.tSignal;    import static pep.Main.tBias;
import static pep.Main.dSignal;    import static pep.Main.dBias;
import static pep.Main.tOnset;     import static pep.Main.tOffset;
import static pep.Main.dOnset;     import static pep.Main.dOffset;
import static pep.Main.cOnset;     import static pep.Main.cOffset;
import static pep.Main.dMask;      import static pep.Main.trialLength;
import static pep.Main.isDegrade;  import static pep.Main.tDegrade;
import static pep.Main.isi;

/**This class is guts of the model, where each run is performed.*/
public class RunTime implements Runnable {
 /**number of variables*/    static int nVariables = Main.numPerVar.length;
 /**attention node*/         Attention attention;
 /**manages goals*/          Executive executive;
 /**number of episodes*/     static int nEpisodes = 1050;
 /**episodic memory*/        Episode[] memory = new Episode[nEpisodes];
 /**target Input nodes*/     Input[] targets = new Input[nTargets];
 /**distracter Input nodes*/ Input[] distracters = new Input[nDistracters];
 /**cue Input nodes*/        Input[] cues = new Input[nCues];
 /**Decision nodes*/         Decision[] decisions = new Decision[nDecisions];
 /**Goal nodes*/             Goal[] goals = new Goal[nGoals];
 /**Response nodes*/         Response[] responses = new Response[nResponses];
 /**episode counter*/        int eNumber;
 /**end of trial marker*/    boolean isTrialOver;
 /**subject number*/         int subj;
 /**condition number*/       int[] condition = new int[nVariables];
 /**target number*/          int[] target = new int[multTarg];
 /**distracter number*/      int[] distracter = new int[multDist];
 /**cue number*/             int[] cue = new int[multCue];
 /**cycle number*/           int cycle;
 /**correct response*/       int correctResp;
 /**actual response*/        int response;
 /**accuracy*/               int accuracy;
 /**stimulus on screen?*/    boolean isStimuli = true;
 /**end of trials*/          private boolean stop;
 /**adjustable signal strength*/
 private double dSigTrend = dSignal;
 /**threshold at response*/  private double threshTest = .45;
 /**amount of practice*/     private int nPractice;
 /**pace of responding*/     private double pace = 2000.0;
 /**thread complete?*/       public boolean deathRow = false;
 /**cycle time resolution*/ private static final int resol = Main.resolution;
 
 /**Sets up the thread.
  * @param subject The subject number.*/
 @SuppressWarnings("empty-statement")
 public RunTime(int subject){
  subj=subject;
 }

 /**Defines the nodes of the model defined and runs the trials.*/
 @Override
 public void run(){
  for(int item=0; item<nTargets; item++)
   targets[item] = new Input(subj,true,false);
  for(int item=0; item<nDistracters; item++)
   distracters[item] = new Input(subj,false,false);
  for(int item=0; item<nCues; item++)
   cues[item] = new Input(subj,false,true);
  Input.clear();
  for(int i=0; i<nEpisodes; i++) memory[i] = new Episode(subj);
  for(int i=0; i<nDecisions; i++) decisions[i] = new Decision(subj);
  Decision.clear();
  for(int i=0; i<nGoals; i++) goals[i] = new Goal(subj);
  Goal.clear();
  for(int i=0; i<nResponses; i++) responses[i] = new Response(subj);
  Response.clear();
  attention = new Attention();
  executive = new Executive();
  pretrainDec(); pretrainRes();
  eNumber = 0;
  setGoals();
  setResponses();
  nPractice = eNumber;

  //---Open I/O Streams---
  StringTokenizer tokenizer = null;
  FileOutputStream out = null;
  PrintStream p = null;
  DataSet trials = new DataSet(Main.source,subj+1);
  try{
   String outFile = "out"+(subj+1)+".txt";
   out = new FileOutputStream(outFile);
   p = new PrintStream(out);
   tokenizer = new StringTokenizer(trials.get(0));
  }
  catch(FileNotFoundException e){
   System.out.println("Error setting up input and output streams.");
  }

  //---Trial Loop---
  stop = false;
  while(!stop){
   correctResp = Integer.parseInt(tokenizer.nextToken());
   for(int c=0; c<multCue; c++)
    cue[c] = Integer.parseInt(tokenizer.nextToken());
   for(int t=0; t<multTarg; t++)
    target[t] = Integer.parseInt(tokenizer.nextToken());
   for(int d=0; d<multDist; d++)
    distracter[d] = Integer.parseInt(tokenizer.nextToken());
   for(int nc=0; nc<nVariables; nc++)
    condition[nc] = Integer.parseInt(tokenizer.nextToken());
   response = 999; accuracy = 999;

   //---Attentional Wandering---
   dSigTrend = attention.wander(dSigTrend,dSignal);

   //---Define Inputs and Run Cycles---
   for(Input i: targets) i.shutOff();
   for(Input i: distracters) i.shutOff();
   for(Input i: cues) i.shutOff();
   isTrialOver = false;
   if(nGoals>1) executive.startTrial();
   else executive.oneGoal();
   for(cycle=1; isTrialOver==false; cycle+=resol){
    if(tOnset<(cycle+resol)&&tOnset>=cycle){
     isStimuli = true;
     for(Input i: targets) i.turnOn(tBias,0);
     for(int i=0; i<multTarg; i++){
      targets[target[i]].turnOn(tBias,tSignal);
      // <-- insert asterisk between slashes to lesion degrade check
      if(isDegrade && condition[nVariables-1]==1)
       targets[target[i]].turnOn(tBias,tDegrade);//*/
     }
    }
    else if(tOffset<cycle) for(Input i: targets) i.remove();
    if(dOnset<(cycle+resol)&&dOnset>=cycle){
     isStimuli = true;
     for(Input i: distracters) i.turnOn(dBias,0);
     for(int i=0; i<multDist; i++)
      distracters[distracter[i]].turnOn(dBias,dSigTrend);
    }
    else if(dMask<(cycle+resol)&&dMask>=cycle)
     for(Input i: distracters) i.mask();
    else if(dOffset<cycle) for(Input i: distracters) i.remove();
    if(cOnset<(cycle+resol)&&cOnset>=cycle){
     isStimuli = true;
     for(Input i: cues) i.turnOn(tBias,0);
     for(int c=0; c<multCue; c++) cues[cue[c]].turnOn(tBias,tSignal);
    }
    else if(cOffset<cycle) for(Input i: cues) i.remove();

    go();
    if(monitorResp()) isTrialOver=true;
    if((cycle-tOnset)>(trialLength-1)) isTrialOver=true;
   }
   for(Input i: targets) i.shutOff();
   for(Input i: distracters) i.shutOff();
   for(Input i: cues) i.shutOff();

   //execute response
   executive.execute();
   for(int ex=0; ex<(200/resol); ex+=resol) go();
   executive.stopSearch();
   cycle-=resol; //correction
   codeTrial();

   //write trial info to file
   p.print(""+(subj+1)+"\t"+response+"\t");
   for(int c=0; c<multCue; c++) p.print(""+cue[c]+"\t");
   for(int nt=0; nt<multTarg; nt++) p.print(target[nt]+"\t");
   for(int k=0; k<multDist; k++) p.print(distracter[k]+"\t");
   for(int m=0; m<nVariables; m++) p.print(condition[m]+"\t");
   p.println(correctResp+"\t"+(cycle-tOnset)+"\t"+accuracy+"\t"+threshTest);

   threshTest=.45;
   response=999;
   memory[eNumber].setRT(cycle-tOnset);

   isStimuli = false;
   for(Input i: targets) i.shutOff();
   for(Input i: distracters) i.shutOff();
   for(Input i: cues) i.shutOff();
   for(int i=0; i<isi; i+=resol) go();
   for(int ep=0; ep<eNumber; ep++) memory[ep].feedback();
   memory[eNumber].setNegativeWeights();

   eNumber++;
   if(eNumber>=nEpisodes) System.out.println("Error: Not enough Episodes.");

   try{
    String nextTrial = trials.get(eNumber-nPractice);
    if(nextTrial.equals("END_OF_FILE")) stop=true;
    else tokenizer = new StringTokenizer(nextTrial);
   }
   catch(Exception e2){
    System.out.println("No END_OF_FILE marker");
    stop=true;
   }
  }

  p.println("END_OF_FILE");
  try{
   p.close(); out.close();
  }
  catch(IOException e3){
   System.out.println("Error closing input and output streams.");
  }
  deathRow = true;
  Main.freeThread.release();
 }


 //-------------------------------------------------------------------
 //------------------------------Methods------------------------------
 //-------------------------------------------------------------------

 /**Performs all the calculations for an individual cycle.*/
 private void go(){
  //---INPUTS---
  for(Input i: distracters){ i.retrieveDist(); i.activate(); }
  for(Input i: targets){ i.retrieveTarg(); i.activate(); }
  for(Input i: cues){ i.retrieveCue(); i.activate(); }
  Input.clear();
  //---EPISODES---
  for(int item=0; item<nDistracters; item++){
   double search = distracters[item].getOut()*Executive.nonSearch.get();
   if(item<nConflicting){
    double targ = targets[item].getOut()*.25*Executive.stimSearch.get();
    search += targ;
   }
   if(search>0){
    for(int ep=0; ep<eNumber; ep++) memory[ep].memberDist(search,item);
   }
   if(isStimuli) memory[eNumber].connectDist(distracters[item].getOut(),item);
  }
  for(int item=0; item<nTargets; item++){
   double search = targets[item].getOut()*Executive.stimSearch.get();
   if(item<nConflicting)
    search += (distracters[item].getOut()*Executive.nonSearch.get()*.25);
   if(search>0){    
    for(int ep=0; ep<eNumber; ep++) memory[ep].memberTarg(search,item);
   }
   if(isStimuli) memory[eNumber].connectTarg(targets[item].getOut(),item);
  }
  for(int item=0; item<nCues; item++){
   double search = cues[item].getOut();
   if(search>0){
    search*=Executive.goalSearch.get();
    for(int ep=0; ep<eNumber; ep++) memory[ep].memberCue(search,item);
    }
   if(isStimuli) memory[eNumber].connectCue(cues[item].getOut(),item);
  }
  for(int item=0; item<nDecisions; item++){
   double search = decisions[item].getOut();
   if(search>0){
    search*=Executive.decSearch.get();
    for(int ep=0; ep<eNumber; ep++) memory[ep].memberDec(search,item);
   }
   if(isStimuli) memory[eNumber].connectDec(decisions[item].getOut(),item);
  }
  for(int g=0; g<nGoals; g++){
   double search = goals[g].getOut();
   if(search>0){
    for(int ep=0; ep<eNumber; ep++) memory[ep].memberGoal(search,g);    
    if(isStimuli) memory[eNumber].connectGoal(goals[g].getOut(),g);
   }
  }
  for(int item=0; item<nResponses; item++){
   double search = responses[item].getOut(threshTest);
   search*=Executive.resSearch.get();
   if(search>0){
    for(int ep=0; ep<eNumber; ep++) memory[ep].memberResp(search,item);
   }
   if(isStimuli)
    memory[eNumber].connectResp(responses[item].getOut(threshTest),item);
  }
  for(int ep=0; ep<eNumber; ep++) memory[ep].activate();
  if(isStimuli) memory[eNumber].activateNew();
  else memory[eNumber].update();
  //---GOALS---
  if(nGoals==1 && Executive.needGoal.get()) goals[0].trigger();
  else if(nGoals==1) goals[0].silence();
  for(Goal i: goals) i.sumAct();
  for(Goal i: goals) i.activate();
  Goal.clear();
  //---DECISIONS---
  for(Decision i: decisions) i.activate();
  Decision.clear();
  //---RESPONSES---
  if(response<nResponses) responses[response].input+=1;
  for(Response i: responses) i.activate();
  Response.clear();
 }

 /**Sets pre-learned input-decision connections.*/
 private void pretrainDec(){
  int[][] idDec = Main.targDec.clone();
  int length = idDec[0].length;
  for(int id=0; id<length; id++)
   targets[idDec[0][id]].overlearnTarg(idDec[1][id]);
  idDec = Main.distDec.clone();
  length = idDec[0].length;
  for(int id=0; id<length; id++)
   distracters[idDec[0][id]].overlearnDist(idDec[1][id]); 
 }

 /**Sets pre-learned decision-response connections.*/
 private void pretrainRes(){
  int[][] decRes = Main.knownRes.clone();
  int length = decRes[0].length;
  for(int id=0; id<length; id++)
   decisions[decRes[0][id]].overlearnResp(decRes[1][id]);
 }

 /**Creates memories of goal instructions.*/
 private void setGoals(){
  if(nGoals>1){
   int[][] aGoal = Main.iCueGoal.clone();
   int length = aGoal[0].length;
   for(int t=0; t<length; t++){
    memory[eNumber].setConnectGoal(aGoal[0][t],aGoal[1][t]); eNumber++;
   }
   aGoal = Main.goalDec.clone();
   length = aGoal[0].length;
   for(int g=0; g<length; g++){
    goals[aGoal[0][g]].connect(aGoal[1][g]);
   }
  }
  else{
   for(int dec=0; dec<nDecisions; dec++) goals[0].connect(dec);
  }
 }

 /**Creates memories of response mappings.*/
 private void setResponses(){
  int[][] binds = Main.iDecRes.clone();
  int length = binds[0].length;
  for(int dec=0; dec<length; dec++){
   memory[eNumber].setConnectRes(binds[0][dec],binds[1][dec]); eNumber++;
  }
  binds = Main.iStimRes.clone();
  length = binds[0].length;
  for(int stim=0; stim<length; stim++){
   memory[eNumber].setConnectSR(binds[0][stim],binds[1][stim]); eNumber++;
  }
 }

 /**Optional method to print instructed mapping weights.*/
 private void printMappings(){
  int[][] binds = Main.iStimRes.clone();
  int length = binds[0].length;
  int id = (nGoals==1) ? 0 : nGoals; //first instructed
  int end = length + id; //last instructed
  double sum = 0.0;
  for(int res=0; id<end; id++,res++){
   sum += memory[id].getWeight(binds[1][res]);
  }
  double average = (double)sum/length;
  System.out.println(average);
 }

 /**Checks for temporal expectancy and generates a temporal bias.
  *@return Temporal expectancy bias.*/
 private double monitorTemp(){
  double expect;
  // <--insert asterisk between slashes to lesion temporal learning 
  double adjustment = 0.0;
  for(int ep=0; ep<eNumber; ep++) adjustment += memory[ep].getRT(pace);
  pace += adjustment;
  double match = (cycle-tOnset)-pace;
  expect = .25;
  expect += Math.pow(match,2)*.00002;
  if(expect>.45) expect=.45;
  //*/expect=.35;
  return expect;
 }

 /**Checks for response identification and ends the trial if necessary.
  *@return If the response has been found.*/
 private boolean monitorResp(){
  boolean found = false;
  double currentThresh = monitorTemp();
  for(int i=0; i<nResponses; i++){
   if(responses[i].activation>currentThresh){
    found = true;
    threshTest = currentThresh;
    response = (response!=999) ? 777 : i; //double response
   }
  }
  return found;
 }

 /**Codes for trial information.*/
 private void codeTrial(){
  if(correctResp==response){
   accuracy = 1;
   pace = pace*.5 + (cycle-tOnset)*.5;
  }
  else{
   accuracy = 0;
   // <--insert asterisk between slashes to lesion error adjustment
   for(int c=0; c<nCues; c++){
    if(memory[eNumber].wasCued(c)){
     for(int heal=0; heal<nPractice; heal++)
      memory[heal].checkInstructCue(c);
    }
   }
   for(int dec=0; dec<nDecisions; dec++){
    if(memory[eNumber].wasDecided(dec)){
     for(int heal=0; heal<nPractice; heal++) 
      memory[heal].checkInstructDec(dec);
    }
   }
   memory[eNumber].tagError();
   //*/
  }
 }
}