package pep;

import java.util.Random;

/**This superclass adds functions for input and output nodes*/
public class SubNode extends Node{
 /**search strength*/                 public double strength;
 /**number of connections*/           public int nConnections;
 /**connection weight array*/         public double weight[];
 /**amount of activation retrieved*/  public double retrieved;
 /**random number generator*/         private final Random r = new Random();

 public SubNode(int subject){
  super(subject);
 }

 /**General method for updating retrieval.
  * @param total Total retrieval activation.
  * @param mult Multiplier.*/
 public void retrieve(double total,double mult){
  if(total>0){
   if(total>1) retrieved /= total; //proportional retrieval
   retrieved -= .1; //small threshold
   retrieved = range(retrieved);
   retrieved *= mult;
   input += retrieved;
   retrieved = 0.0;
  }
 }

 /**Sets pre-learned target-decision connections.
  * @param node Node number.*/
 public void overlearnTarg(int node){
  weight[node] = 6.0;
 }

 /**Sets pre-learned distracter-decision connections.
  * @param node Node number.*/
 public void overlearnDist(int node){
  weight[node] = 1.0;
 }

 /**Sets pre-learned decision-response connections.
  * @param node Node number.*/
 public void overlearnResp(int node){
  weight[node] = 3.0;
 }

 /**Calculates and returns output to Episode nodes.
  * @return Output to episodes.*/
 public double getOut(){
  double output = range(activation-threshold)*strength;
  return output;
 }

 /**Calculates and returns output to Episode nodes.
  * @param thresh Dynamic response threshold.
  * @return Output to episodes.*/
 public double getOut(double thresh){
  double output = range(activation-thresh)*strength;
  return output;
 }

 /**Gaussian number generator.
  * @param rate Magnitude of numbers.
  * @return Gaussian distributed number.*/
 public double addNoise(double rate){
  double n = (range((r.nextGaussian()*.2)+.5)*rate);
  return n;
 }
}