package pep;

/**Class for switching between goals (e.g., cue and target identification).*/
public class Executive {
 /**goal search strength*/
 public static ThreadLocal<Double> goalSearch = new ThreadLocal<>();
 /**goal found truth value (for maintaining identified goal)*/
 public static ThreadLocal<Boolean> needGoal = new ThreadLocal<>();
 /**decision found truth value (for maintaining identified decision)*/
 public static ThreadLocal<Boolean> needDec = new ThreadLocal<>();
 /**stimulus search strength*/
 public static ThreadLocal<Double> stimSearch = new ThreadLocal<>();
 /**decision search strength*/
 public static ThreadLocal<Double> decSearch = new ThreadLocal<>();
 /**response search strength*/
 public static ThreadLocal<Double> resSearch = new ThreadLocal<>();
 /**non-target search strength*/
 public static ThreadLocal<Double> nonSearch = new ThreadLocal<>();
 /**currently focused information*/     private static double focus;
 /**currently non-focused information*/ private static double nonfocus;
 /**currently non-focused goal*/        private static double goalNF;
 /**currently non-focused stimulus*/    private static double stimNF;

 /**Sets up Executive goal manager.*/
 public Executive(){
  focus = 1.0;
  nonfocus = .15;
  goalNF = Evolve.sample[19];
  stimNF = Evolve.sample[20];
  nonSearch.set(nonfocus);
 }

 /**Sets up parameters for start of trial (more than one goal).*/
 public void startTrial(){
  needGoal.set(true);
  needDec.set(false);
  goalSearch.set(focus);
  stimSearch.set(nonfocus);
  decSearch.set(nonfocus);
  resSearch.set(nonfocus);
 }

 /**Start decision/response search.*/
 public void foundGoal(){
  needGoal.set(false);
  needDec.set(true);
  goalSearch.set(goalNF);
  stimSearch.set(focus);
  decSearch.set(focus);
  resSearch.set(focus);
 }

  /**For one goal tasks.*/
 public void oneGoal(){
  needGoal.set(true);
  needDec.set(false);
  goalSearch.set(focus);
  stimSearch.set(focus);
  decSearch.set(focus);
  resSearch.set(focus);
 }

 public void foundDec(){
  needGoal.set(false);
  needDec.set(false);
  goalSearch.set(goalNF);
  stimSearch.set(stimNF);
  decSearch.set(focus);
  resSearch.set(focus);
 }

 /**Start response execution.*/
 public void execute(){
  needGoal.set(false);
  needDec.set(false);
  goalSearch.set(0.0);
  stimSearch.set(0.0);
  decSearch.set(0.0);
  resSearch.set(Evolve.sample[21]);
 }

 /**Stops all searches (end of trial).*/
 public void stopSearch(){
  needGoal.set(false);
  needDec.set(false);
  goalSearch.set(0.0);
  stimSearch.set(0.0);
  decSearch.set(0.0);
  resSearch.set(0.0);
 }
}