package pep;

/**This class defines the operations of Goal nodes.*/
public class Goal extends SubNode{
 /**total amount memory retrieved*/
 public static ThreadLocal<Double> totRetrieved = new ThreadLocal<>();
 /**total amount of activation*/
 private static final ThreadLocal<Double> totAct = new ThreadLocal<>();

 /**Creates a new Decision node.
  * @param subject Subject number used for static variable array.*/
 public Goal(int subject){
  super(subject);
  decay = resolutionAdjust(Evolve.sample[9]);
  threshold = Evolve.sample[10];
  strength = Evolve.sample[11];
  nConnections = Main.nDecisions;
  weight = new double[nConnections];
 }

 /**Updates activation for each cycle and searches responses.*/
 public void activate(){
  retrieve();
  input -= ((totAct.get()-activation)*.75);
  input += addNoise(Evolve.sample[12]);
  update();
  goalCheck();
  double output = activation - threshold;
  if(output>0)
   for(int d=0; d<nConnections; d++)
    Main.runs[subj].decisions[d].prepare(output*weight[d]);
 }

 /**Adds to total input for interference calculation.*/
 public void sumAct(){
  double total = totAct.get() + activation;
  totAct.set(total);
 }

 /**Adds retrieved activation to input with SubNode method.*/
 private void retrieve(){
  double total = totRetrieved.get();
  double multiplier = Executive.goalSearch.get();
  retrieve(total,multiplier);
 }

 /**Special method to trigger goal with single-goal tasks.*/
 public void trigger(){
  retrieved = .6;
  totRetrieved.set(retrieved);
 }

 /**Special method to silence goal with single-goal tasks.*/
 public void silence(){
  retrieved = 0;
  totRetrieved.set(0.0);
 }

 /**Connect Goal node to Decision node.
  * @param node Node to connect to.*/
 public void connect(int node){
  weight[node] = Evolve.sample[13];
 }

 /**Clears totRetrieved and totInput for the next cycle.*/
 public static void clear(){
  totAct.set(0.0);
  totRetrieved.set(0.0);
 }

 /**Maintains goal once discovered.*/
 public void goalCheck(){
  if(activation>.5 & Executive.needGoal.get())
   Main.runs[subj].executive.foundGoal();
 }
}