package pep;

/**This superclass defines the basic fields and methods of Nodes*/
public class Node{
 /**rate of decay*/                   public double decay;
 /**incoming activation*/             public double input;
 /**current activation*/              public double activation;
 /**threshold to pass on activation*/ public double threshold;
 /**subject number*/                  public int subj;

 /**Creates a new Node.
  * @param subject Subject number used for static variable array.*/
 public Node(int subject){
  subj=subject;
  decay = resolutionAdjust(Evolve.sample[14]);
  input = 0.0;
  activation = 0.0;
 }

/**Uses logistic input to update the activation of the Node.*/
 public void update(){
  input = (1 / (1 + Math.exp(-((input*8)-4))));
  activation = activation*(1-decay) + input*decay;
  input = 0.0;
 }

 /**Ranges activation between 0 and 1.
  * @param in Value to be ranged.
  * @return The newly ranged value.*/
 public double range(double in){
  return (in<0.0) ? 0.0 : (in>1.0 ? 1.0 : in);
 }

 /**Ranges activation between 0 and max.
  * @param in Value to be ranged.
  * @param max Maximum value to return.
  * @return The newly ranged value.*/
 public double range(double in,double max){
  return (in<0.0) ? 0.0 : (in>max ? max : in);
 }

 /**Adjusts parameter based on resolution adjustment.
  * @param value Raw parameter.
  * @return Adjusted parameter.*/
 public static double resolutionAdjust(double value){
  double adjustedValue = 0.0;
  for(int r=0; r<Main.resolution; r++)
   adjustedValue=(value+(adjustedValue*(1-value)));
  return adjustedValue;
 }
}